@@ -670,10 +670,11 @@ socket_connected (SoupSocket *sock, GSocketConnection *conn, GError *error)
 
 	if (priv->connect_cancel) {
 		GCancellable *cancellable = priv->connect_cancel;
+		gboolean cancelled = g_cancellable_is_cancelled (cancellable);
 
 		g_object_unref (priv->connect_cancel);
 		priv->connect_cancel = NULL;
-		if (g_cancellable_is_cancelled (cancellable))
+		if (cancelled)
 			return SOUP_STATUS_CANCELLED;
 	}
 
@@ -1242,8 +1243,10 @@ soup_socket_disconnect (SoupSocket *sock)
 	 */
 	g_object_ref (sock);
 
-	/* Give all readers a chance to notice the connection close */
-	g_signal_emit (sock, signals[READABLE], 0);
+	if (priv->non_blocking) {
+		/* Give all readers a chance to notice the connection close */
+		g_signal_emit (sock, signals[READABLE], 0);
+	}
 
 	/* FIXME: can't disconnect until all data is read */
 
@@ -1278,6 +1281,9 @@ soup_socket_is_connected (SoupSocket *sock)
  *
  * Returns the #SoupAddress corresponding to the local end of @sock.
  *
+ * Calling this method on an unconnected socket is considered to be
+ * an error, and produces undefined results.
+ *
  * Return value: (transfer none): the #SoupAddress
  **/
 SoupAddress *
@@ -1293,13 +1299,25 @@ soup_socket_get_local_address (SoupSocket *sock)
 		GSocketAddress *addr;
 		struct sockaddr_storage sa;
 		gssize sa_len;
+		GError *error = NULL;
 
-		addr = g_socket_get_local_address (priv->gsock, NULL);
+		if (priv->gsock == NULL) {
+			g_warning ("%s: socket not connected", G_STRLOC);
+			goto unlock;
+		}
+
+		addr = g_socket_get_local_address (priv->gsock, &error);
+		if (addr == NULL) {
+			g_warning ("%s: %s", G_STRLOC, error->message);
+			g_error_free (error);
+			goto unlock;
+		}
 		sa_len = g_socket_address_get_native_size (addr);
 		g_socket_address_to_native (addr, &sa, sa_len, NULL);
 		priv->local_addr = soup_address_new_from_sockaddr ((struct sockaddr *)&sa, sa_len);
 		g_object_unref (addr);
 	}
+unlock:
 	g_mutex_unlock (&priv->addrlock);
 
 	return priv->local_addr;
@@ -1311,6 +1329,9 @@ soup_socket_get_local_address (SoupSocket *sock)
  *
  * Returns the #SoupAddress corresponding to the remote end of @sock.
  *
+ * Calling this method on an unconnected socket is considered to be
+ * an error, and produces undefined results.
+ *
  * Return value: (transfer none): the #SoupAddress
  **/
 SoupAddress *
@@ -1326,13 +1347,25 @@ soup_socket_get_remote_address (SoupSocket *sock)
 		GSocketAddress *addr;
 		struct sockaddr_storage sa;
 		gssize sa_len;
+		GError *error = NULL;
 
-		addr = g_socket_get_remote_address (priv->gsock, NULL);
+		if (priv->gsock == NULL) {
+			g_warning ("%s: socket not connected", G_STRLOC);
+			goto unlock;
+		}
+
+		addr = g_socket_get_remote_address (priv->gsock, &error);
+		if (addr == NULL) {
+			g_warning ("%s: %s", G_STRLOC, error->message);
+			g_error_free (error);
+			goto unlock;
+		}
 		sa_len = g_socket_address_get_native_size (addr);
 		g_socket_address_to_native (addr, &sa, sa_len, NULL);
 		priv->remote_addr = soup_address_new_from_sockaddr ((struct sockaddr *)&sa, sa_len);
 		g_object_unref (addr);
 	}
+unlock:
 	g_mutex_unlock (&priv->addrlock);
 
 	return priv->remote_addr;
